import 'dart:async';
import 'dart:convert';
import 'dart:math';

import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';

import 'window_manager_platform_interface.dart';

const MethodChannel _channel = MethodChannel('plugins.rive.app/window_manager');

class MethodChannelWindowManager extends WindowManagerPlatform {
  static final Random _random = Random.secure();
  
  @override
  Future<bool> createWindow(String key, {Offset offset, Size size}) async {
    await _channel.invokeMethod(
      'createWindow',
      {
        "key": key,
        "x": offset?.dx,
        "y": offset?.dy,
        "width": size?.width,
        "height": size?.height,
      },
    );
    var _key = await lastWindowKey();
    return _key == key;
  }

  @override
  Future<bool> resizeWindow(String key, Size size) async {
    return _channel.invokeMethod<bool>(
      'resizeWindow',
      {
        "key": key,
        "width": size?.width,
        "height": size?.height,
      },
    );
  }

  @override
  Future<bool> moveWindow(String key, Offset offset) async {
    return _channel.invokeMethod<bool>(
      'moveWindow',
      {
        "key": key,
        "x": offset?.dx,
        "y": offset?.dy,
      },
    );
  }

  @override
  Future<int> keyIndex(String key) {
    return _channel.invokeMethod<int>(
      'keyIndex',
      {
        "key": key,
      },
    );
  }

  @override
  Future<int> windowCount() {
    return _channel.invokeMethod<int>('windowCount');
  }

  @override
  Future<bool> closeWindow(String key) {
    try {
      return _channel.invokeMethod<bool>(
        'closeWindow',
        {
          "key": key,
        },
      );
    } catch (e) {
      return Future.value(false);
    }
  }

  @override
  Future<String> lastWindowKey() {
    return _channel.invokeMethod<String>("lastWindowKey");
  }

  @override
  Future<Map> getWindowStats(String key) {
    return _channel.invokeMethod<Map>(
      "getWindowStats",
      {
        "key": key,
      },
    );
  }

  @override
  Future<Size> getWindowSize(String key) async {
    final _stats = await getWindowStats(key);
    final w = _stats['width'] as double;
    final h = _stats['height'] as double;
    return Size(w, h);
  }

  @override
  Future<Offset> getWindowOffset(String key) async {
    final _stats = await getWindowStats(key);
    final x = _stats['offsetX'] as double;
    final y = _stats['offsetY'] as double;
    return Offset(x, y);
  }

  @override
  String generateKey([int length = 10]) {
    var values = List<int>.generate(length, (i) => _random.nextInt(256));
    return base64Url.encode(values);
  }
}
