import 'dart:async';

import 'package:flutter/widgets.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';

import 'method_channel_window_manager.dart';

abstract class WindowManagerPlatform extends PlatformInterface {
  /// Constructs a WindowManagerPlatform.
  WindowManagerPlatform() : super(token: _token);

  static final Object _token = Object();

  static WindowManagerPlatform _instance = MethodChannelWindowManager();

  /// The default instance of [WindowManagerPlatform] to use.
  ///
  /// Defaults to [MethodChannelUrlLauncher].
  static WindowManagerPlatform get instance => _instance;

  /// Platform-specific plugins should set this with their own platform-specific
  /// class that extends [WindowManagerPlatform] when they register themselves.
  // TODO(amirh): Extract common platform interface logic.
  // https://github.com/flutter/flutter/issues/43368
  static set instance(WindowManagerPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  Future<bool> createWindow(String key, {Offset offset, Size size}) async {
    throw UnimplementedError('createWindow() has not been implemented.');
  }

  Future<bool> resizeWindow(String key, Size size) async {
    throw UnimplementedError('resizeWindow() has not been implemented.');
  }

  Future<bool> moveWindow(String key, Offset offset) async {
    throw UnimplementedError('moveWindow() has not been implemented.');
  }

  Future<int> keyIndex(String key) {
    throw UnimplementedError('keyIndex() has not been implemented.');
  }

  Future<int> windowCount() {
    throw UnimplementedError('windowCount() has not been implemented.');
  }

  Future<bool> closeWindow(String key) {
    throw UnimplementedError('closeWindow() has not been implemented.');
  }

  Future<String> lastWindowKey() {
    throw UnimplementedError('canLaunch() has not been implemented.');
  }

  Future<Map> getWindowStats(String key) {
    throw UnimplementedError('lastWindowKey() has not been implemented.');
  }

  Future<Size> getWindowSize(String key) async {
    throw UnimplementedError('getWindowSize() has not been implemented.');
  }

  Future<Offset> getWindowOffset(String key) async {
    throw UnimplementedError('getWindowOffset() has not been implemented.');
  }

  String generateKey([int length = 10]) {
    throw UnimplementedError('generateKey() has not been implemented.');
  }
}
